@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014 - 2015 Alberts Muktupāvels
+ * Copyright (C) 2014-2016 Alberts Muktupāvels
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "libdisplay-config/flashback-display-config.h"
 #include "libend-session-dialog/gf-end-session-dialog.h"
 #include "libidle-monitor/flashback-idle-monitor.h"
+#include "libinput-settings/gf-input-settings.h"
 #include "libinput-sources/gf-input-sources.h"
 #include "libnotifications/gf-notifications.h"
 #include "libpolkit/flashback-polkit.h"
@@ -40,7 +41,7 @@
 #include "libworkarounds/gf-workarounds.h"
 
 #ifdef WITH_LIBSTATUS_NOTIFIER
-#include "libwatcher/gf-watcher.h"
+#include <libstatus-notifier/sn.h>
 #endif
 
 struct _GfApplication
@@ -62,6 +63,7 @@ struct _GfApplication
   GfBluetoothApplet      *bluetooth;
   GfDesktopBackground    *background;
   GfEndSessionDialog     *dialog;
+  GfInputSettings        *input_settings;
   GfInputSources         *input_sources;
   GfNotifications        *notifications;
   GfPowerApplet          *power;
@@ -72,7 +74,7 @@ struct _GfApplication
   GfWorkarounds          *workarounds;
 
 #ifdef WITH_LIBSTATUS_NOTIFIER
-  GfWatcher              *watcher;
+  SnWatcher              *watcher;
 #endif
 };
 
@@ -160,6 +162,7 @@ settings_changed (GSettings   *settings,
   SETTING_CHANGED (bluetooth, "bluetooth-applet", gf_bluetooth_applet_new)
   SETTING_CHANGED (background, "desktop-background", gf_desktop_background_new)
   SETTING_CHANGED (dialog, "end-session-dialog", gf_end_session_dialog_new)
+  SETTING_CHANGED (input_settings, "input-settings", gf_input_settings_new)
   SETTING_CHANGED (input_sources, "input-sources", gf_input_sources_new)
   SETTING_CHANGED (notifications, "notifications", gf_notifications_new)
   SETTING_CHANGED (power, "power-applet", gf_power_applet_new)
@@ -169,11 +172,26 @@ settings_changed (GSettings   *settings,
   SETTING_CHANGED (sound, "sound-applet", gf_sound_applet_new)
   SETTING_CHANGED (workarounds, "workarounds", gf_workarounds_new)
 
+#undef SETTING_CHANGED
+
 #ifdef WITH_LIBSTATUS_NOTIFIER
-  SETTING_CHANGED (watcher, "watcher", gf_watcher_new)
+  if (key == NULL || g_strcmp0 (key, "watcher") == 0)
+    {
+      if (g_settings_get_boolean (settings, "watcher"))
+        {
+          if (application->watcher == NULL)
+            application->watcher = sn_watcher_new (SN_WATCHER_FLAGS_NONE);
+        }
+      else
+        {
+          g_clear_object (&application->watcher);
+        }
+    }
 #endif
 
-#undef SETTING_CHANGED
+  if (application->input_settings)
+    gf_input_settings_set_display_config (application->input_settings,
+                                          application->config);
 
   if (application->shell)
     flashback_shell_set_display_config (application->shell, application->config);
