@@ -67,7 +67,7 @@ power_save_mode_changed (MetaDBusDisplayConfig *skeleton,
   /* If DPMS is unsupported, force the property back. */
   if (manager->power_save_mode == META_POWER_SAVE_UNSUPPORTED)
     {
-      meta_dbus_display_config_set_power_save_mode (META_DBUS_DISPLAY_CONFIG (manager), META_POWER_SAVE_UNSUPPORTED);
+      meta_dbus_display_config_set_power_save_mode (skeleton, META_POWER_SAVE_UNSUPPORTED);
       return;
     }
 
@@ -184,19 +184,15 @@ static char *
 make_display_name (FlashbackMonitorManager *manager,
                    MetaOutput              *output)
 {
+  MetaConnectorType type;
   char *inches = NULL;
   char *vendor_name = NULL;
   char *ret;
 
-  switch (output->connector_type)
-    {
-    case META_CONNECTOR_TYPE_LVDS:
-    case META_CONNECTOR_TYPE_eDP:
-      ret = g_strdup (_("Built-in display"));
-      goto out;
-    default:
-      break;
-    }
+  type = output->connector_type;
+
+  if (type == META_CONNECTOR_TYPE_eDP || type == META_CONNECTOR_TYPE_LVDS)
+    return g_strdup (_("Built-in display"));
 
   if (output->width_mm > 0 && output->height_mm > 0)
     {
@@ -236,7 +232,6 @@ make_display_name (FlashbackMonitorManager *manager,
       ret = g_strdup (vendor_name);
     }
 
- out:
   g_free (inches);
   g_free (vendor_name);
 
@@ -305,7 +300,7 @@ handle_get_resources (MetaDBusDisplayConfig *skeleton,
 
   g_variant_builder_init (&crtc_builder, G_VARIANT_TYPE ("a(uxiiiiiuaua{sv})"));
   g_variant_builder_init (&output_builder, G_VARIANT_TYPE ("a(uxiausauaua{sv})"));
-  g_variant_builder_init (&mode_builder, G_VARIANT_TYPE ("a(uxuud)"));
+  g_variant_builder_init (&mode_builder, G_VARIANT_TYPE ("a(uxuudu)"));
 
   for (i = 0; i < manager->n_crtcs; i++)
     {
@@ -384,6 +379,8 @@ handle_get_resources (MetaDBusDisplayConfig *skeleton,
                              g_variant_new_string (get_connector_type_name (output->connector_type)));
       g_variant_builder_add (&properties, "{sv}", "underscanning",
                              g_variant_new_boolean (output->is_underscanning));
+      g_variant_builder_add (&properties, "{sv}", "supports-underscanning",
+                             g_variant_new_boolean (output->supports_underscanning));
 
       edid = flashback_monitor_manager_read_edid (manager, output);
 
@@ -395,6 +392,20 @@ handle_get_resources (MetaDBusDisplayConfig *skeleton,
           g_bytes_unref (edid);
         }
 
+      if (output->tile_info.group_id)
+        {
+          g_variant_builder_add (&properties, "{sv}", "tile",
+                                 g_variant_new ("(uuuuuuuu)",
+                                                output->tile_info.group_id,
+                                                output->tile_info.flags,
+                                                output->tile_info.max_h_tiles,
+                                                output->tile_info.max_v_tiles,
+                                                output->tile_info.loc_h_tile,
+                                                output->tile_info.loc_v_tile,
+                                                output->tile_info.tile_w,
+                                                output->tile_info.tile_h));
+        }
+
       g_variant_builder_add (&output_builder, "(uxiausauaua{sv})",
                              i, /* ID */
                              (gint64) output->winsys_id,
@@ -412,12 +423,13 @@ handle_get_resources (MetaDBusDisplayConfig *skeleton,
 
       mode = &manager->modes[i];
 
-      g_variant_builder_add (&mode_builder, "(uxuud)",
+      g_variant_builder_add (&mode_builder, "(uxuudu)",
                              i, /* ID */
                              (gint64) mode->mode_id,
                              (guint32) mode->width,
                              (guint32) mode->height,
-                             (double) mode->refresh_rate);
+                             (double) mode->refresh_rate,
+                             (guint32) mode->flags);
     }
 
   meta_dbus_display_config_complete_get_resources (skeleton,
@@ -453,7 +465,7 @@ handle_apply_configuration (MetaDBusDisplayConfig *skeleton,
   int y;
   int new_screen_width;
   int new_screen_height;
-  guint transform;
+  gint transform;
   guint output_index;
   GPtrArray *crtc_infos;
   GPtrArray *output_infos;
@@ -974,8 +986,6 @@ flashback_display_config_class_init (FlashbackDisplayConfigClass *config_class)
 static void
 flashback_display_config_init (FlashbackDisplayConfig *config)
 {
-  MetaDBusDisplayConfig *display_config;
-
   config->skeleton = meta_dbus_display_config_skeleton_new ();
   config->manager = flashback_monitor_manager_new (config->skeleton);
   config->bus_name = g_bus_own_name (G_BUS_TYPE_SESSION,
@@ -996,6 +1006,12 @@ flashback_display_config_new (void)
                        NULL);
 }
 
+/**
+ * flashback_display_config_get_monitor_manager:
+ * @config: a #FlashbackMonitorManager
+ *
+ * Returns: (transfer none):
+ */
 FlashbackMonitorManager *
 flashback_display_config_get_monitor_manager (FlashbackDisplayConfig *config)
 {
