@@ -653,7 +653,11 @@ meta_monitor_config_assign_crtcs (MetaConfiguration       *config,
     }
 
   all_outputs = flashback_monitor_manager_get_outputs (manager, &n_outputs);
-  g_assert (n_outputs == config->n_outputs);
+  if (n_outputs != config->n_outputs)
+    {
+      g_hash_table_destroy (assignment.info);
+      return FALSE;
+    }
 
   for (i = 0; i < n_outputs; i++)
     {
@@ -2050,18 +2054,19 @@ flashback_monitor_config_restore_previous (FlashbackMonitorConfig *config)
       /* The user chose to restore the previous configuration. In this
        * case, restore the previous configuration. */
       MetaConfiguration *prev_config = config_ref (config->previous);
-      apply_configuration (config, prev_config);
+      gboolean ok = apply_configuration (config, prev_config);
       config_unref (prev_config);
 
       /* After this, self->previous contains the rejected configuration.
        * Since it was rejected, nuke it. */
       g_clear_pointer (&config->previous, (GDestroyNotify) config_unref);
+
+      if (ok)
+        return;
     }
-  else
-    {
-      if (!flashback_monitor_config_apply_stored (config))
-        flashback_monitor_config_make_default (config);
-    }
+
+  if (!flashback_monitor_config_apply_stored (config))
+    flashback_monitor_config_make_default (config);
 }
 
 gboolean
