@@ -47,6 +47,18 @@
  * for the reasoning */
 #define DPI_FALLBACK 96.0
 
+/* Array index matches MetaMonitorTransform */
+static gfloat transform_matrices[][6] = {
+  {  1,  0,  0,  0,  1,  0 }, /* normal */
+  {  0, -1,  1,  1,  0,  0 }, /* 90° */
+  { -1,  0,  1,  0, -1,  1 }, /* 180° */
+  {  0,  1,  0, -1,  0,  1 }, /* 270° */
+  { -1,  0,  1,  0,  1,  0 }, /* normal flipped */
+  {  0,  1,  0,  1,  0,  0 }, /* 90° flipped */
+  {  1,  0,  0,  0, -1,  1 }, /* 180° flipped */
+  {  0, -1,  1, -1,  0,  1 }, /* 270° flipped */
+};
+
 struct _FlashbackMonitorManagerPrivate
 {
   Display               *xdisplay;
@@ -73,6 +85,53 @@ enum
 
 static GParamSpec *object_properties[N_PROPERTIES] = { NULL, };
 
+enum
+{
+  MONITORS_CHANGED,
+
+  LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
+static gboolean
+calculate_viewport_matrix (FlashbackMonitorManager *manager,
+                           MetaOutput              *output,
+                           gfloat                   viewport[6])
+{
+  gfloat x, y, width, height;
+
+  if (!output->crtc)
+    return FALSE;
+
+  x = (float) output->crtc->rect.x / manager->screen_width;
+  y = (float) output->crtc->rect.y / manager->screen_height;
+  width  = (float) output->crtc->rect.width / manager->screen_width;
+  height = (float) output->crtc->rect.height / manager->screen_height;
+
+  viewport[0] = width;
+  viewport[1] = 0.0f;
+  viewport[2] = x;
+  viewport[3] = 0.0f;
+  viewport[4] = height;
+  viewport[5] = y;
+
+  return TRUE;
+}
+
+static inline void
+multiply_matrix (float a[6],
+                 float b[6],
+                 float res[6])
+{
+  res[0] = a[0] * b[0] + a[1] * b[3];
+  res[1] = a[0] * b[1] + a[1] * b[4];
+  res[2] = a[0] * b[2] + a[1] * b[5] + a[2];
+  res[3] = a[3] * b[0] + a[4] * b[3];
+  res[4] = a[3] * b[1] + a[4] * b[4];
+  res[5] = a[3] * b[2] + a[4] * b[5] + a[5];
+}
+
 static void
 add_monitor (FlashbackMonitorManager *manager,
              MetaMonitorInfo         *monitor)
@@ -1313,6 +1372,7 @@ read_current_config (FlashbackMonitorManager *manager)
       mode->width = xmode->width;
       mode->height = xmode->height;
       mode->refresh_rate = (xmode->dotClock / ((float)xmode->hTotal * xmode->vTotal));
+      mode->flags = xmode->modeFlags;
       mode->name = get_xmode_name (xmode);
     }
 
@@ -1559,6 +1619,10 @@ flashback_monitor_manager_class_init (FlashbackMonitorManagerClass *manager_clas
 
   g_object_class_install_properties (object_class, N_PROPERTIES,
                                      object_properties);
+
+  signals[MONITORS_CHANGED] =
+    g_signal_new ("monitors-changed", FLASHBACK_TYPE_MONITOR_MANAGER,
+                  G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_NONE, 0);
 }
 
 static void
@@ -2096,6 +2160,21 @@ flashback_monitor_manager_on_hotplug (FlashbackMonitorManager *manager)
     flashback_monitor_config_make_default (manager->monitor_config);
 }
 
+gboolean
+flashback_monitor_manager_get_monitor_matrix (FlashbackMonitorManager *manager,
+                                              MetaOutput              *output,
+                                              gfloat                   matrix[6])
+{
+  gfloat viewport[9];
+
+  if (!calculate_viewport_matrix (manager, output, viewport))
+    return FALSE;
+
+  multiply_matrix (viewport, transform_matrices[output->crtc->transform], matrix);
+
+  return TRUE;
+}
+
 MetaOutput *
 flashback_monitor_manager_get_outputs (FlashbackMonitorManager *manager,
                                        unsigned int            *n_outputs)
@@ -2193,6 +2272,7 @@ flashback_monitor_manager_rebuild_derived (FlashbackMonitorManager *manager)
     }
 
   g_signal_emit_by_name (manager->priv->display_config, "monitors-changed");
+  g_signal_emit (manager, signals[MONITORS_CHANGED], 0);
 
   g_free (old_monitor_infos);
 }
