@@ -62,9 +62,6 @@ GtkAdjustment *     gvc_channel_bar_get_adjustment      (GvcChannelBar *bar);
 gboolean            gvc_channel_bar_get_is_muted        (GvcChannelBar *bar);
 void                gvc_channel_bar_set_is_muted        (GvcChannelBar *bar,
                                                          gboolean       is_muted);
-gboolean            gvc_channel_bar_get_show_mute       (GvcChannelBar *bar);
-void                gvc_channel_bar_set_show_mute       (GvcChannelBar *bar,
-                                                         gboolean       show_mute);
 void                gvc_channel_bar_set_size_group      (GvcChannelBar *bar,
                                                          GtkSizeGroup  *group,
                                                          gboolean       symmetric);
